@@ -24,16 +24,26 @@ class MSData():
         If logger is pssed all methods of MSData will log in the activity.
     """
 
-    def __init__(self, datareader, logger=None):
-        self.datareader = datareader
+    def __init__(self, datareader=None, logger=None):
         self.logger = logger
-        if self.logger is not None:
-            self.logger.info(f'Reading data {self.datareader.filename}.')
-        self.data = self.datareader()
-        self.data.index = list(map(float, self.data.index))
-        self.time = np.array(self.data.index)
-        self.matrix = self.data.values
-        self.isotope_names = np.array(self.data.columns)
+        if datareader is not None:
+            self.datareader = datareader
+            self.data = self.datareader()
+            self.data.index = list(map(float, self.data.index))
+            self.time = np.array(self.data.index)
+            self.matrix = self.data.values
+            self.isotope_names = np.array(self.data.columns)
+            if self.logger is not None:
+                self.logger.info(f'Reading data {self.datareader.filename}.')
+        else:
+            self.datareader = None
+            self.data = None
+            self.time = None
+            self.matrix = None
+            self.isotope_names = []
+            if self.logger is not None:
+                self.logger.info(f'Creating empty MSData.')
+
         self.isotopes = {}
         self.param = None
         self.selector = None
@@ -41,7 +51,7 @@ class MSData():
         self.laser_off, self.laser_on = None, None
         self.names = None
         self.srms = None
-        self.average_peaks = None
+        self.means = None
         self.quantified = None
         self.lod = None
         self.corrected_IS = None
@@ -191,7 +201,7 @@ class MSData():
             self.data.plot(ax=ax, y=el, kind='line', legend=False)
         else:
             self.data.plot(ax=ax, kind='line', legend=True)
-            ax.legend(loc='upper center', bbox_to_anchor=(0.5, -0.05),
+            ax.legend(loc='upper center', bbox_to_anchor=(0.5, -0.075),
                       fancybox=True, shadow=True, ncol=10)
 
         if logax:
@@ -276,6 +286,11 @@ class MSData():
 
         self.quantified = pd.DataFrame()
         for el, isotope in self.isotopes.items():
+
+            if el not in self.srms.columns:
+                if self.logger is not None:
+                    self.logger.error(f'Missing srm {el}.')
+
             isotope.quantify(srm_name=srm_name)
             self.quantified[el] = isotope.quantified
 
@@ -355,6 +370,10 @@ class MSData():
             ts_coef = pd.ExcelFile(
                 './default_sum_koef.xlsx').parse(0, index_col=0, header=None).to_dict()[1]
 
+        if self.logger is not None:
+            self.logger.info(
+                f'Using these coeficients for total sum: {ts_coef}.')
+
         self.corrected_TS = self.quantified.copy()
         self.corrected_TS.drop(skip_isotopes, axis=1, inplace=True)
 
@@ -483,7 +502,22 @@ class MSData():
         for el, isotope in self.isotopes.items():
             isotope.elemental_distribution(despiked, bcgcor_method, dx, dy)
 
-    def export_matrices(self, path):
+    def quantify_maps(self, slopes, intercepts):
+        """
+        Calculate quantification for elemental distribution matrix for all isotopes.
+
+        Parameters
+        ----------
+        intercepts : dict
+            Dict of intercepts for each isotope.
+        slopes : dict
+            Dict of slopes for each isotope.
+        """
+
+        for el, isotope in self.isotopes.items():
+            isotope.elmap.quantify_map(slopes[el], intercepts[el])
+
+    def export_matrices(self, path, quantified=False):
         """
         Export all elemental distribution data to excel file. Each isotope 
         will be a matrix on one sheet.
@@ -496,8 +530,12 @@ class MSData():
 
         writer = pd.ExcelWriter(path, engine='xlsxwriter')
         for el, isotope in self.isotopes.items():
-            df = pd.DataFrame(isotope.elmap.matrix,
-                              index=isotope.elmap.y, columns=isotope.elmap.x)
+            if quantified is False:
+                df = pd.DataFrame(isotope.elmap.matrix,
+                                  index=isotope.elmap.y, columns=isotope.elmap.x)
+            else:
+                df = pd.DataFrame(isotope.elmap.qmap,
+                                  index=isotope.elmap.y, columns=isotope.elmap.x)
             df.to_excel(writer, sheet_name=el)
         writer.save()
 
@@ -532,12 +570,14 @@ class Isotope():
         `data` interpreted as an array.
     """
 
-    def __init__(self, isotope_name, ms_data, logger=None):
+    def __init__(self, isotope_name, ms_data=None, logger=None):
         self.isotope_name = elem_resolution(isotope_name)
-        self.ms_data = ms_data
+        if ms_data is not None:
+            self.ms_data = ms_data
+            self.data = ms_data.data[isotope_name].values
+            self.time = ms_data.data.index
         self.isotope_number = int(
             ''.join([c for c in self.isotope_name if c.isnumeric()]))
-        self.data = ms_data.data[isotope_name].values
         self.despiked = None
         self.bcg = None
         self.bcg_corrected = None
@@ -546,7 +586,7 @@ class Isotope():
         self.ratio = None
         self.quantified = None
         self.elmap = None
-        self.logger = loggers
+        self.logger = logger
 
     def __len__(self):
         return len(self.data)
@@ -618,9 +658,8 @@ class Isotope():
                     'Bacground correction requires selection of peaks first.')
             return
 
-        if self.bcgcor_method != method:
-            self.bcg = Background(
-                self, self.ms_data.laser_on, self.ms_data.laser_off)
+        self.bcg = Background(
+            self, self.ms_data.laser_on, self.ms_data.laser_off)
 
         if not despiked:
             data = np.copy(self.data)
@@ -661,9 +700,11 @@ class Isotope():
         laser_on = self.ms_data.laser_on
 
         if bcgcor_method is not None:
-            if self.bcg_corrected is None:
+            if self.bcg_corrected is None or self.bcgcor_method != bcgcor_method:
                 self.bcg_correction(method=bcgcor_method, despiked=despiked)
-            data = np.copy(self.bcg_corrected)
+                data = np.copy(self.bcg_corrected)
+            else:
+                data = np.copy(self.bcg_corrected)
         else:
             if not despiked:
                 data = np.copy(self.data)
@@ -678,7 +719,7 @@ class Isotope():
                 means.append(np.mean(data[on[0]:on[1]]))
             elif method == 'integral':
                 sample_y = data[on[0]:on[1]]
-                sample_x = data.index[on[0]:on[1]]
+                sample_x = self.time[on[0]:on[1]]
                 means.append(np.trapz(sample_y, sample_x))
             else:
                 if self.logger is not None:
@@ -749,6 +790,12 @@ class Isotope():
                 list(itertools.chain.from_iterable(self.bcg.bcg_all)))
         elif scale == 'beginning':
             bcg = np.array(self.bcg.bcg_all[0])
+        elif scale == 'end':
+            bcg = np.array(self.bcg.bcg_all[-1])
+        else:
+            if self.logger is not None:
+                self.logger.error(f'Unknown scale: {scale}.')
+
         if method == 'integral':
             self.lod = (bcg.std()*ablation_time*self.ratio)
         elif method == 'intensity':
@@ -847,9 +894,11 @@ class ElementalMap():
         self.matrix = matrix
         self.dx = dx
         self.dy = dy
-        self.create_xy()
+        self.qmap = None
+        if self.matrix is not None and self.dx is not None and self.dy is not None:
+            self.create_xy()
 
-    def __call__(self, fig=None, ax=None, units='', axis=True, clb=True, *args, **kwargs):
+    def __call__(self, fig=None, ax=None, units='', axis=True, clb=True, quantified=False, title='', *args, **kwargs):
         """
         Show image of elemental map.
         Parameters
@@ -858,26 +907,36 @@ class ElementalMap():
             Axes where plot will be drawn, if None new axes will be created.
         units : str
             Label for colorbar
-        axis : boolean
-            If true then show axis with labels in the image.
-        clb : boolean
-            Wheather to show colorbal in image.
+        axis : boolean (Optional)
+            If true then show axis with labels in the image. Default is True.
+        clb : boolean (Optional)
+            Wheather to show colorbal in image. Default is True.
+        quantified : bool (optional)
+            Whether to plot intensities or quantified map. Default is False.
+        title: str (Optional)
+            Figure title to be used. Default is no title.
         """
         if fig is None or ax is None:
             fig, ax = plt.subplots()
         ax.cla()
-
-        im = plt.imshow(self.matrix, extent=[
-                        0, self.x[-1], self.y[-1], 0], *args, **kwargs)
+        if quantified is False:
+            im = ax.imshow(self.matrix, extent=[
+                0, self.x[-1], self.y[-1], 0], *args, **kwargs)
+        else:
+            im = ax.imshow(self.qmap, extent=[
+                0, self.x[-1], self.y[-1], 0], *args, **kwargs)
 
         if not axis:
             ax.axis('off')
+
         if clb:
             divider = make_axes_locatable(ax)
             cax = divider.append_axes("right", size="5%", pad=0.05)
             clb = fig.colorbar(im, cax=cax)
             clb.ax.set_title(units)
 
+        fig.suptitle(title)
+
     def __repr__(self):
         return f'{self.__class__.__name__}: {self.matrix.shape}'
 
@@ -946,3 +1005,17 @@ class ElementalMap():
         """
 
         self.matrix = np.flip(self.matrix, axis)
+
+    def quantify_map(self, slope, intercept=0):
+        """
+        Quantify elemental map.
+
+        Parameters
+        ----------
+        slope: float
+            Slope of linear regression for quantification.
+        intercept: float
+            Intercept of linear regression for quantification.
+        """
+
+        self.qmap = (self.matrix-intercept)/slope
